﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;

using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;

namespace Platform.Logging.DatabaseLogger
{
    [ProviderAlias("Database")]
    public class PlatformDatabaseLoggerProvider : PlatformBatchLoggerProvider<PlatformDatabaseLoggerOptions>
    {
        public PlatformDatabaseLoggerProvider(IConfigurationSection configuration)
            : base(configuration)
        {
            _ProcessId = Process.GetCurrentProcess().Id.ToString();
        }

        protected override string SectionName => "Database";

        private string _ProcessId;
        private string _ServerName;
        private string _SqlInsertCommand;
        private string _SqlDeleteCommand;
        private string _SqlConnectionString;
        private SqlConnection _SqlConnection;

        internal override PlatformBatchLogger<PlatformDatabaseLoggerOptions> CreateLoggerImplementation(string categoryName)
        {
            return new PlatformDatabaseLogger(this, categoryName);
        }

        protected override void OnConfigurationChanged()
        {
            // call base class
            base.OnConfigurationChanged();

            // create directory
            try
            {
                // extract connection data
                _SqlConnectionString = ProviderOptions.ConnectionString;
                if (string.IsNullOrEmpty(_SqlConnectionString))
                    _SqlConnectionString = "Data Source=.;Initial Catalog=Logging;Integrated Security=true";

                // get insert command
                _SqlInsertCommand = ProviderOptions.InsertCommand;
                if (string.IsNullOrEmpty(_SqlInsertCommand))
                    _SqlInsertCommand =
                        "INSERT INTO Loggings( [ServerName], [LoggingDate], [ProcessId], [ThreadId], [Category], [Level], [EventId], [EventName], [Message], [Exception] )" +
                        " VALUES( @servername, @date, @processid, @threadid, @category, @level, @eventid, @eventname @message, @exception )";

                // get insert command
                _SqlDeleteCommand = ProviderOptions.DeleteCommand;
                if (string.IsNullOrEmpty(_SqlDeleteCommand))
                    _SqlDeleteCommand =
                        "DELETE FROM Loggings WHERE [LoggingDate] < @data;";

                // gets the server name
                if (string.IsNullOrEmpty(ProviderOptions.ServerName)) _ServerName = Environment.MachineName;
                else _ServerName = ProviderOptions.ServerName;

                // get connection
                _SqlConnection = null;
            }
            catch (Exception ex)
            {
                SimpleFileLogger.WriteLine(Configuration, "{0} Logger Failed to read setting: {1}", SectionName, ex.Message);
            }
        }

        protected async override Task WriteMessagesAsync(IEnumerable<PlatformBatchLoggerMessage> messages, CancellationToken cancellationToken)
        {
            try
            {
                if (string.IsNullOrEmpty(_SqlConnectionString))
                    return;

                // get new connection
                if (_SqlConnection == null)
                    _SqlConnection = new SqlConnection(_SqlConnectionString);

                // open the connection
                if (_SqlConnection.State != ConnectionState.Open)
                    await _SqlConnection.OpenAsync();

                // add messages
                foreach( var message in messages )
                {
                    // create new command
                    using (var command = _SqlConnection.CreateCommand())
                    {
                        command.CommandTimeout = 240;
                        command.CommandType = CommandType.Text;
                        command.CommandText = _SqlInsertCommand;

                        command.Parameters.Add(new SqlParameter("@servername", _ServerName));
                        command.Parameters.Add(new SqlParameter("@date", message.Timestamp));
                        command.Parameters.Add(new SqlParameter("@processid", _ProcessId));
                        command.Parameters.Add(new SqlParameter("@threadid", message.ThreadId));
                        command.Parameters.Add(new SqlParameter("@category", message.Category ));
                        command.Parameters.Add(new SqlParameter("@level", (int)message.LogLevel));
                        command.Parameters.Add(new SqlParameter("@eventid", message.EventId.Id));
                        command.Parameters.Add(new SqlParameter("@eventname", message.EventId.Name));
                        command.Parameters.Add(new SqlParameter("@message", message.Message));
                        command.Parameters.Add(new SqlParameter("@exception", message.Exception));
                        await command.ExecuteNonQueryAsync();
                    }
                }
            }
            catch (Exception ex)
            {
                SimpleFileLogger.WriteLine(Configuration, "{0} Logger Failed to write messages: {1}", SectionName, ex.Message);
            }
        }

        protected async override Task CleanupLoggingAsync(int retention)
        {
            try
            {
                if (string.IsNullOrEmpty(_SqlConnectionString))
                    return;

                // get new connection
                if (_SqlConnection == null)
                    _SqlConnection = new SqlConnection(_SqlConnectionString);

                // open the connection
                if (_SqlConnection.State != ConnectionState.Open)
                    await _SqlConnection.OpenAsync();

                // create new command
                using (var command = _SqlConnection.CreateCommand())
                {
                    command.CommandTimeout = 240;
                    command.CommandType = CommandType.Text;
                    command.CommandText = _SqlDeleteCommand;
                    command.Parameters.Add(new SqlParameter("@date", DateTime.Now - TimeSpan.FromDays(retention)));
                    command.ExecuteNonQuery();
                }
            }
            catch (Exception ex)
            {
                SimpleFileLogger.WriteLine(Configuration, "{0} Logger Failed to delete messages: {1}", SectionName, ex.Message);
            }            
        }
    }
}
